"""SelfLearners Class for calling "self_learners" API endpoints."""
import requests
from restfly.endpoint import APIEndpoint


class SelfLearners(APIEndpoint):
    """
    "SelfLearners" API endpoint
    """

    _path = "/self_learners"

    def create(
        self,
        payload: dict,
    ) -> requests.Response:
        return self._post(json=payload, verify=False)



"""Reports Class for calling "reports" API endpoints."""
import time

from restfly.endpoint import APIEndpoint


class Reports(APIEndpoint):
    """
    "reports" API endpoint

    API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Usage
    """

    _path = "reports"
    _query_delay = 5  # sleep seconds.

    def get_report(self, **kwargs) -> dict:
        """
        Get all user records based on query parameters sent in kwargs.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Usage

        Returns:
             list of all user accounts, per set params in kwargs.
        """
        # Post the Report request.
        self._path = f"{self._path}.json"

        result = self._post(json=kwargs).json()

        # Query the API to check if the report is built.
        self._path = f"{self._path.split('.')[0]}/{result['id']}.json"
        while result["ready"] is not True:
            print(f"Ready: {result['ready']} for Job ID: {result['id']}.")
            result = self._get().json()
            time.sleep(kwargs.get("query_delay", self._query_delay))
        return result



"""Environments Class for calling "configurations" API endpoints."""
from restfly.endpoint import APIEndpoint

from skytapsdk.helpers import links_iterator


class Environments(APIEndpoint):
    """
    "configurations" API endpoint

    API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Configur
    """

    _path = "configurations"

    def get_all(self, **kwargs) -> list:
        """
        Get all environment records based on query parameters sent in kwargs.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Getallenvironments

        Returns:
            list of all environments, per set params in kwargs.
        """
        return links_iterator(api=self._api, path=self._path, **kwargs)

    def get_by_id(self, uid: int) -> dict:
        """
        GET a specific environment.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Getenvironment

        Args:
            uid: ID of desired environment.

        Returns:
            dict of queried environment.
        """
        self._path = f"{self._path}/{uid}"
        return self._get().json()



from .assets import Assets
from .environments import Environments
from .projects import Projects
from .reports import Reports
from .templates import Templates
from .users import Users

__all__ = [
    "Assets",
    "Environments",
    "Projects",
    "Reports",
    "Templates",
    "Users",
]



"""Events Class for calling "events" API endpoints."""
import requests
from restfly.endpoint import APIEndpoint


class Events(APIEndpoint):
    """
    "Events" API endpoint
    """

    _path = "events"

    def get(
        self,
        event_id: str = None,
    ) -> requests.Response:
        if event_id:
            self._path = f"{self._path}/{event_id}"
        return self._get(verify=False)

    def get_participants(
        self,
        event_id: str,
        participant_id: str = None,
    ) -> requests.Response:
        pass  # FIXME: This doesn't work yet.
        # self._path = f"{self._path}/{event_id}"
        # parameters = {
        #     "tab": "participants-tab",
        # }
        # return self._get(verify=False, params=parameters,)

        # self._path = f"{self._path}/{event_id}/students"
        # if participant_id:
        #     self._path = f"{self._path}/{participant_id}"
        # return self._get(verify=False)

    def update_participants(
        self,
        event_id: str,
        participant_id: str,
        payload: dict,
    ) -> requests.Response:
        self._path = f"{self._path}/{event_id}/event_participants/{participant_id}"
        return self._patch(
            json=payload,
            verify=False,
        )



import logging

from restfly.session import APISession

from .skytap_endpoints import (Assets, Environments, Projects, Reports,
                               Templates, Users)


class Skytap(APISession):
    """A controller to access Endpoints in the Skytap API."""

    BASE_URL = "https://cloud.skytap.com"
    API_VERSION = 1

    def __init__(
        self,
        username: str = None,
        password: str = None,
        token: str = None,
        **kwargs,
    ):
        """
        Create a Skytap object for managing interactions with the Skytap API.

        Args:
            username: Username for API auth
            password: Password for API auth (optional)
            token: Token for API auth (preferred over password)
            kwargs: Various settable values
                api_version (int) Defaults to self.API_VERSION.
        """

        self._username = username
        self._password = password
        self._token = token
        self._api_version = self.API_VERSION
        self._url = self.BASE_URL

        # Use setter to update the self._api_version AND rewrite the self._url.
        self.api_version = kwargs.get("api_version", self.API_VERSION)

        super(Skytap, self).__init__(
            username=self._username,
            password=self._password,
            token=self._token,
            **kwargs,
        )
        self.login()

    def login(self):
        if self._username:
            if self._token:
                self._session.auth = (
                    self._username,
                    self._token,
                )
            elif self._password:
                self._session.auth = (
                    self._username,
                    self._password,
                )
            else:
                logging.warning("Token or Password is required for authentication.")
        self._session.headers.update(
            {
                "Accept": "application/json",
                "Content-Type": "application/json",
            }
        )

    def logout(self):
        self._session.auth = None

    @property
    def api_version(self) -> int:
        return self._api_version

    @api_version.setter
    def api_version(self, version: int):
        """When api_version is updated you need to also update the self._url."""
        self._api_version = version
        self._url = self.BASE_URL
        if self._api_version == 2:
            self._url = f"{self._url}/v2"

    @property
    def users(self) -> Users:
        return Users(self)

    @property
    def configurations(self) -> Environments:
        return Environments(self)

    @property
    def environments(self) -> Environments:
        """
        Some people might find it easier to use 'configurations' by the name 'environments' since that is what they see in
        the UI.
        """
        return Environments(self)

    @property
    def templates(self) -> Templates:
        return Templates(self)

    @property
    def assets(self) -> Assets:
        return Assets(self)

    @property
    def reports(self) -> Reports:
        return Reports(self)

    @property
    def projects(self) -> Projects:
        return Projects(self)



from restfly import APIEndpoint, APISession


def links_iterator(api: APISession, path: str = "", **kwargs) -> list:
    """If API returns paginated results that use a 'Links' header.  Iterate over those to get all."""
    _count = kwargs.get("count", 100)
    _offset = kwargs.get("offset", 0)
    _params = {
        "count": _count,
        "offset": _offset,
    }
    _params.update(kwargs)

    response = api.get(path=path, params=_params)
    results = response.json()
    while "link" in response.headers and 'rel="next"' in response.headers["Link"]:
        _params["offset"] = _params["offset"] + _count
        response = api.get(path=path, params=_params)
        results += response.json()
    return results


class LinksIterator(APIEndpoint):
    """If API returns paginated results that use a 'Links' header.  Iterate over those to get all."""

    COUNT: int = 100
    OFFSET: int = 0

    def get_results(self, count: int = COUNT, offset: int = OFFSET, **kwargs) -> list:
        params = {
            "count": count,
            "offset": offset,
        }
        params.update(kwargs)
        response = self._api.get(path=self._path, params=params)
        results = response.json()
        while "link" in response.headers and 'rel="next"' in response.headers["Link"]:
            params["offset"] = params["offset"] + count
            response = self._api.get(path=self._path, params=params)
            results += response.json()
        return results



import logging

from restfly.session import APISession

from .course_manager_endpoints import Events, SelfLearners


class CourseManager(APISession):
    """A controller to access Endpoints in the Skytap Course Manager API."""

    BASE_URL = "https://zscaler.skytap-portal.com/api/v1"

    def __init__(
        self,
        token: str = None,
        secret: str = None,
        **kwargs,
    ):
        """
        Create a Skytap object for managing interactions with the Skytap API.

        Args:
            token: Username for API auth
            secret: Token for API auth (preferred over password)
            kwargs: Various settable values
                api_version (int) Defaults to self.API_VERSION.
        """

        self._token = token
        self._secret = secret
        self._url = self.BASE_URL

        super(CourseManager, self).__init__(
            token=self._token,
            secret=self._secret,
            **kwargs,
        )
        self.login()

    def login(self) -> None:
        self._session.auth = (
            self._token,
            self._secret,
        )

    def logout(self):
        self._session.auth = None

    @property
    def self_learners(self) -> SelfLearners:
        return SelfLearners(self)

    @property
    def events(self) -> Events:
        return Events(self)



"""Users Class for calling "users" API endpoints."""
from restfly.endpoint import APIEndpoint

from skytapsdk.helpers import links_iterator


class Users(APIEndpoint):
    """
    "users" API endpoint

    API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#User2
    """

    _path = "users"

    def get_all(self, **kwargs) -> list:
        """
        Get all user records based on query parameters sent in kwargs.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Listusers

        Returns:
            list of all user accounts, per set params in kwargs.
        """
        return links_iterator(api=self._api, path=self._path, **kwargs)

    def get_by_id(self, uid: int) -> dict:
        """
        GET a specific user.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#User2

        Args:
            uid: ID of desired user.

        Returns:
            dict of queried user account.
        """
        self._path = f"{self._path}/{uid}"
        return self._get().json()

    def add_user(
        self,
        **kwargs,
    ):
        """Add a user"""
        # Add parameters to payload
        payload = {}
        for key, value in kwargs.items():
            payload[key] = value

        return self._post(json=payload)



from setuptools import find_packages, setup

__version = "20230414.0"
__author = "Dax Mickelson"
__author_email = "dmickelson@zscaler.com"
__license = "BSD"
__name = "skytapsdk"
__description = "SDK for interacting with Skytap's APIs."
__long_description = __description
__url = ""
__classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Plugins",
    "Intended Audience :: Developers",
    "Intended Audience :: Other Audience",
    "License :: OSI Approved :: BSD License",
    "Natural Language :: English",
    "Operating System :: POSIX :: Linux",
    "Operating System :: MacOS",
    "Operating System :: Microsoft",
    "Programming Language :: Python :: 3",
    "Topic :: System :: Systems Administration",
    "Topic :: Utilities",
]
__keywords = "skytap"
__packages = find_packages(exclude=["docs", "tests*"])
__install_requires = ["restfly"]
__python_requires = ">=3.6"
__package_data = {}
__data_files = []

setup(
    name=__name,
    version=__version,
    description=__description,
    long_description=__long_description,
    url=__url,
    author=__author,
    author_email=__author_email,
    license=__license,
    classifiers=__classifiers,
    keywords=__keywords,
    packages=__packages,
    install_requires=__install_requires,
    python_requires=__python_requires,
    package_data=__package_data,
    data_files=__data_files,
)



"""Environments Class for calling "configurations" API endpoints."""
import logging

from restfly.endpoint import APIEndpoint

from skytapsdk.helpers import links_iterator


class Projects(APIEndpoint):
    """
    "projects" API endpoint

    API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Project
    """

    _path = "projects"

    def get_all(self, **kwargs) -> list:
        """
        Get all project records based on query parameters sent in kwargs.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Listprojects

        Returns:
            list of all projects, per set params in kwargs.
        """
        return links_iterator(api=self._api, path=self._path, **kwargs)

    def get_by_id(self, uid: int) -> dict:
        """
        GET a specific project.

        API Doc Reference: https://help.skytap.com/API_v2_Documentation.html#Getprojectdescription

        Args:
            uid: ID of desired project.

        Returns:
            dict of queried project.
        """
        self._path = f"{self._path}/{uid}.json"
        return self._get().json()

    def delete_by_id(self, uid: int) -> dict:
        """
        DELETE a specific project.

        API Doc Reference: https://help.skytap.com/API_Documentation.html#Deleteproject

        Args:
            uid: ID of desired project.

        Returns:
            dict of result.
        """
        logging.warning(
            "The delete method for Projects is an API Version 1 only feature."
        )
        self._path = f"{self._path}/{uid}"
        return self._delete()
